DECLARE FUNCTION NextByte% ()
DECLARE SUB LoadPCX (f$)

DIM SHARED PaletteBuffer AS STRING * 768
DIM SHARED Buffer AS STRING * 512
DIM SHARED Header AS STRING * 128
DIM SHARED nextPointer

  IF COMMAND$ <> "" THEN
    FileName$ = COMMAND$
  ELSE
    INPUT "Enter filename:", FileName$
  END IF

  SCREEN 13
  CALL LoadPCX(FileName$)
  a$ = INPUT$(1)
  SCREEN 0, 0, 0
  WIDTH 80
  SYSTEM

'The following is a dummy error handler used for trapping a "file not found"
'error.
ErrorHandler:
RESUME NEXT

SUB LoadPCX (f$)
' This loads in the actual PCX-file and displays it.
 
  f$ = UCASE$(f$)
  IF INSTR(f$, ".PCX") = 0 THEN f$ = f$ + ".PCX"

  'Enable error checking
  ON ERROR GOTO ErrorHandler
  'Open the PCX file.
  OPEN f$ FOR BINARY AS 1
  'File not found if ERR contains code 53
  IF ERR = 53 THEN EXIT SUB
  'Disable error checking
  ON ERROR GOTO 0

  'Read in header information
  Header = INPUT$(128, 1)
  Manufacturer$ = MID$(Header, 1, 1)
  Version = ASC(MID$(Header, 2, 1))
  Encoding = ASC(MID$(Header, 3, 1))
  BitsPerPixel = ASC(MID$(Header, 4, 1))
  xMin% = ASC(MID$(Header, 5, 1)) + ASC(MID$(Header, 6, 1)) * &H100
  yMin% = ASC(MID$(Header, 7, 1)) + ASC(MID$(Header, 8, 1)) * &H100
  xMax% = ASC(MID$(Header, 9, 1)) + ASC(MID$(Header, 10, 1)) * &H100
  yMax% = ASC(MID$(Header, 11, 1)) + ASC(MID$(Header, 12, 1)) * &H100
 
  'Check to see whether we can display it
  IF Version <> 5 THEN
    'If other version than 5, don't display
    CLOSE 1
    EXIT SUB
  END IF
  IF (xMax% > 319) OR (xMin% < 0) OR (yMax% > 199) OR (yMin% < 0) THEN
    'If it doesn't fit on-screen, don't display
    CLOSE 1
    EXIT SUB
  END IF

  'Load in the palette
  SEEK 1, LOF(1) - 768
  'Read in identifier
  DataByte% = ASC(INPUT$(1, 1))

  IF DataByte% <> 12 THEN
    'If there is no palette, don't display
    CLOSE 1
    EXIT SUB
  END IF

  PaletteBuffer = INPUT$(768, 1)
  'Set palette - this is slow!!
  FOR Counter% = 0 TO 255
    Red% = ASC(MID$(PaletteBuffer, Counter% * 3 + 1, 1)) \ 4
    Green% = ASC(MID$(PaletteBuffer, Counter% * 3 + 2, 1)) \ 4
    Blue% = ASC(MID$(PaletteBuffer, Counter% * 3 + 3, 1)) \ 4
    PALETTE Counter%, Red% + Green% * &H100 + Blue% * &H10000
  NEXT

  'Go back to start of graphic data
  SEEK 1, 129

  x% = xMin%
  y% = yMin%
  xMax% = xMax% + 1
  yMax% = yMax% + 1

  'Decode and display graphics
  WHILE (y% < yMax%)
    'Read next byte
    DataByte% = NextByte%

    'Reset counter
    HowMany% = 1

    'If it is encoded, extract the count information and read in the colour byte
    IF (DataByte% AND &HC0) = &HC0 THEN
      HowMany% = DataByte% AND &H3F
      DataByte% = NextByte%
    END IF

    'Display it
    FOR Counter% = 1 TO HowMany%
      'Display Pixel
      PSET (x%, y%), DataByte%
      'Next pixel
      x% = x% + 1

      'If End of Line reached, next line
      IF x% = xMax% THEN
        y% = y% + 1
        x% = xMin%
      END IF

    NEXT
  WEND
  CLOSE 1
END SUB

FUNCTION NextByte%
  'Check to see whether next block should be read
  IF nextPointer < 1 THEN
    'If so, read it and reset the pointer
    Buffer = INPUT$(512, 1)
    nextPointer = 1
  END IF

  'Return byte in buffer
  NextByte = ASC(MID$(Buffer, nextPointer, 1))

  'Adjust pointer
  nextPointer = nextPointer + 1
  IF nextPointer > 512 THEN nextPointer = 0
END FUNCTION

